/*
 * Copyright (C) 2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */


struct filter_chain_info {
	unsigned int channels;
	unsigned int samplerate;
	struct list_head filters;
	char *inbuf, *outbuf;
	size_t *in_loaded, *out_loaded;
};

struct filter_node {
	struct filter *filter;
	struct list_head node;
	void *private_data;
	char *buf;
	size_t bufsize, loaded;
	int argc;
	char **argv;
};

#define DECLARE_EXTERN_FILTER_INIT(name) extern int name ## _init(struct filter *f)
#define FILTER_INIT(filter) {.name=#filter, .init = filter ## _init, .parse_config = NULL, .print_help = NULL},

/* filters that are always present */
DECLARE_EXTERN_FILTER_INIT(wav);
DECLARE_EXTERN_FILTER_INIT(compress);

/* next the optional filters */
#ifdef HAVE_MAD_H
DECLARE_EXTERN_FILTER_INIT(mp3dec);
#define MP3DEC_FILTER FILTER_INIT(mp3dec)
#else
#define MP3DEC_FILTER
#endif

#ifdef HAVE_OGGVORBIS
DECLARE_EXTERN_FILTER_INIT(oggdec);
#define OGGDEC_FILTER FILTER_INIT(oggdec)
#else
#define OGGDEC_FILTER
#endif

/* a macro that defines an array of all available filters */
#define DEFINE_FILTER_ARRAY(fa) struct filter fa[] = { \
	FILTER_INIT(wav) \
	FILTER_INIT(compress) \
	MP3DEC_FILTER \
	OGGDEC_FILTER \
	{ .name = NULL } };

void close_filters(struct filter_chain_info *fci);
int filter_io(struct filter_chain_info *fci);
int filter_init(struct filter *all_filters);
struct filter_node *check_filter_arg(char *filter_arg);

struct filter {
	const char *name;
	int (*init)(struct filter *f);
	int (*open)(struct filter_node *fn);
	ssize_t (*convert)(char *inbuf, size_t len, struct filter_node *fn,
		struct filter_chain_info *fci);
	void (*close)(struct filter_node *fn);
	void (*print_help)(void);
	int (*parse_config)(struct filter_node *fn);
};
